@@ -1,7 +1,7 @@
 #
-# $Id: Base.pm,v 2.2 2003/09/02 21:20:58 wpm Exp $
+# $Id$
 #
-# (c) 2003 Morgan Stanley and Co.
+# (c) 2003-2004 Morgan Stanley and Co.
 # See ..../src/LICENSE for terms of distribution.
 #
 
@@ -13,12 +13,13 @@ use strict;
 use English;
 use Carp;
 use File::Basename qw(basename);
+use Date::Format;
 
 use IO::File;
 use IO::Pipe;
 
 our $AUTOLOAD	= "";
-our $VERSION = '1.1';
+our $VERSION = '1.9';
 
 our %Carp =
   (
@@ -53,12 +54,17 @@ sub new {
 
     my $self = {};
 
-    foreach my $key ( qw( localtime noauth localauth encrypt quiet ) ) {
+    foreach my $key ( qw( localtime noauth localauth encrypt quiet timestamps ) ) {
 	$self->{$key}++ if $args{$key};
     }
 
     # AFS::Command::VOS -> vos
-    $self->{command} = $args{command} || lc((split(/::/,$class))[2]);
+    if ( $args{command} ) {
+        my @commands = (split /\s+/,$args{command});
+        push (@{$self->{command}},@commands);
+    } else {
+        @{$self->{command}} = lc((split(/::/,$class))[2]);
+    }
 
     bless $self, $class;
 
@@ -83,7 +89,7 @@ sub supportsArgument {
     my $argument = shift;
     return unless $self->_operations($operation);
     return unless $self->_arguments($operation);
-    return exists $self->{_arguments}->{$operation}->{$argument}
+    return exists $self->{_arguments}->{$operation}->{$argument};
 }
 
 sub _Carp {
@@ -107,45 +113,76 @@ sub _operations {
 
 	my %operations = ();
 
-	my $pipe = IO::Pipe->new() || do {
-	    $self->_Carp("Unable to create pipe: $ERRNO\n");
-	    return;
-	};
+	#
+	# This hack is necessary to support the offline/online "hidden"
+	# vos commands.  These won't show up in the normal help output,
+	# so we have to check for them individually.  Since offline and
+	# online are implemented as a pair, we can just check one of
+	# them, and assume the other is there, too.
+	#
 
-	my $pid = fork();
+	foreach my $type ( qw(default hidden) ) {
 
-	unless ( defined $pid ) {
-	    $self->_Carp("Unable to fork: $ERRNO\n");
-	    return;
-	}
+	    if ( $type eq 'hidden' ) {
+		next unless $self->isa("AFS::Command::VOS");
+	    }
 
-	if ( $pid == 0 ) {
-
-	    STDOUT->fdopen( $pipe->writer()->fileno(), "w" ) || do {
-		$self->_Croak("Unable to redirect stdout: $ERRNO\n");
+	    my $pipe = IO::Pipe->new() || do {
+		$self->_Carp("Unable to create pipe: $ERRNO\n");
+		return;
 	    };
 
-	    exec $self->{command}, 'help';
-	    die "Unable to exec $self->{command} help: $ERRNO\n";
+	    my $pid = fork();
 
-	} else {
-	    $pipe->reader();
-	    while ( defined($_ = $pipe->getline()) ) {
-		next if /Commands are:/;
-		my ($command) = split;
-		next if $command =~ /^(apropos|help)$/;
-		$operations{$command}++;
+	    unless ( defined $pid ) {
+		$self->_Carp("Unable to fork: $ERRNO\n");
+		return;
 	    }
-	}
 
-	unless ( waitpid($pid,0) ) {
-	    $self->_Carp("Unable to get status of child process ($pid)");
-	    return;
-	}
+	    if ( $pid == 0 ) {
+
+		STDERR->fdopen( STDOUT->fileno(), "w" ) ||
+		  $self->_Croak("Unable to redirect stderr: $ERRNO\n");
+		STDOUT->fdopen( $pipe->writer()->fileno(), "w" ) ||
+		  $self->_Croak("Unable to redirect stdout: $ERRNO\n");
+
+		if ( $type eq 'default' ) {
+		    exec @{$self->{command}}, 'help';
+		} else {
+		    exec @{$self->{command}}, 'offline', '-help';
+		}
+		die "Unable to exec @{$self->{command}} help: $ERRNO\n";
+
+	    } else {
+
+		$pipe->reader();
+
+		while ( defined($_ = $pipe->getline()) ) {
+		    if ( $type eq 'default' ) {
+			next if /Commands are:/;
+			my ($command) = split;
+			next if $command =~ /^(apropos|help)$/;
+			$operations{$command}++;
+		    } else {
+			if ( /^Usage:/ ) {
+			    $operations{offline}++;
+			    $operations{online}++;
+			}
+		    }
+		}
+
+	    }
+
+	    unless ( waitpid($pid,0) ) {
+		$self->_Carp("Unable to get status of child process ($pid)");
+		return;
+	    }
+
+	    if ( $? ) {
+		$self->_Carp("Error running @{$self->{command}} help.  Unable to configure $class");
+		return;
+	    }
 
-	if ( $? ) {
-	    $self->_Carp("Error running $self->{command} help.  Unable to configure $class");
-	    return;
 	}
 
 	$self->{_operations} = \%operations;
@@ -165,12 +202,14 @@ sub _arguments {
       {
        optional		=> {},
        required		=> {},
+       aliases		=> {},
       };
 
-    my $command = $self->{command};
+    my @command;
+    push (@command, @{$self->{command}});
 
     unless ( $self->_operations($operation) ) {
-	$self->_Carp("Unsupported $command operation '$operation'\n");
+	$self->_Carp("Unsupported @command operation '$operation'\n");
 	return;
     }
 
@@ -193,10 +232,12 @@ sub _arguments {
 
     if ( $pid == 0 ) {
 
+	STDERR->fdopen( STDOUT->fileno(), "w" ) ||
+	  die "Unable to redirect stderr: $ERRNO\n";
 	STDOUT->fdopen( $pipe->writer()->fileno(), "w" ) ||
 	  die "Unable to redirect stdout: $ERRNO\n";
-	exec $command, 'help', $operation;
-	die "Unable to exec $command help $operation: $ERRNO\n";
+	exec @command, $operation, '-help';
+	die "Unable to exec @command help $operation: $ERRNO\n";
 
     } else {
 
@@ -204,7 +245,13 @@ sub _arguments {
 
 	while ( <$pipe> ) {
 
-	    next unless s/^Usage:.*$command\s+$operation\s+//;
+	    if ( /Unrecognized operation '$operation'/ ) {
+		$self->_Carp("Unsupported @command operation '$operation'\n");
+		$errors++;
+		last;
+	    }
+
+	    next unless s/^Usage:.*\s+$operation\s+//;
 
 	    while ( $_ ) {
 		if ( s/^\[\s*-(\w+?)\s*\]\s*//  ) {
@@ -221,7 +268,7 @@ sub _arguments {
 		} elsif ( s/^\s*-(\w+?)\s*// ) {
 		    $arguments->{required}->{$1} = 0;
 		} else {
-		    $self->_Carp("Unable to parse $command help for $operation\n" .
+		    $self->_Carp("Unable to parse @command help for $operation\n" .
 				 "Unrecognized string: '$_'");
 		    $errors++;
 		    last;
@@ -234,13 +281,31 @@ sub _arguments {
 
     }
 
+    #
+    # XXX -- Hack Alert!!!
+    #
+    # Because some asshole decided to change the force option to vos
+    # release from -f to -force, you can't use the API tranparently
+    # with 2 different vos binaries that support the 2 different options.
+    #
+    # If we need more of these, we can add them, as this let's us
+    # alias one argument to another.
+    #
+    if ( $self->isa("AFS::Command::VOS") && $operation eq 'release' ) {
+	if ( exists $arguments->{optional}->{f} ) {
+	    $arguments->{aliases}->{force} = 'f';
+	} elsif ( exists $arguments->{optional}->{force} ) {
+	    $arguments->{aliases}->{f} = 'force';
+	}
+    }
+
     unless ( waitpid($pid,0) ) {
 	$self->_Carp("Unable to get status of child process ($pid)");
 	$errors++;
     }
 
     if ( $? ) {
-	$self->_Carp("Error running $command help $operation.  Unable to configure $command $operation");
+	$self->_Carp("Error running @command $operation -help.  Unable to configure @command $operation");
 	$errors++;
     }
 
@@ -258,7 +323,7 @@ sub _save_stderr {
 	return;
     };
 
-    my $command = basename($self->{command});
+    my $command = basename((split /\s+/,@{$self->{command}})[0]);
 
     $self->{tmpfile} = "/tmp/.$command.$self->{operation}.$$";
 
@@ -347,7 +412,12 @@ sub _parse_arguments {
 
     } else {
 
-	my @argv = ( $self->{command}, $self->{operation} );
+	my @argv = ( @{$self->{command}}, $self->{operation} );
+
+	foreach my $key ( keys %args ) {
+	    next unless $arguments->{aliases}->{$key};
+	    $args{$arguments->{aliases}->{$key}} = delete $args{$key};
+	}
 
 	foreach my $key ( qw( noauth localauth encrypt ) ) {
 	    next unless $self->{$key};
@@ -374,20 +444,21 @@ sub _parse_arguments {
 		    next unless exists $args{$key};
 		}
 
-		push(@argv,"-$key");
-
 		if ( $hasvalue ) {
 		    if ( ref $args{$key} eq 'HASH' || ref $args{$key} eq 'ARRAY' ) {
 			unless ( ref $hasvalue eq 'ARRAY' ) {
 			    $self->_Carp("Invalid argument '$key': can't provide a list of values");
 			    return;
 			}
+			push(@argv,"-$key");
 			foreach my $value ( ref $args{$key} eq 'HASH' ? %{$args{$key}} : @{$args{$key}} ) {
 			    push(@argv,$value);
 			}
 		    } else {
-			push(@argv,$args{$key});
+			push(@argv,"-$key",$args{$key});
 		    }
+		} else {
+		    push(@argv,"-$key") if $args{$key};
 		}
 
 		delete $args{$key};
@@ -489,6 +560,7 @@ sub _parse_output {
     $self->{errors} = "";
 
     while ( defined($_ = $self->{handle}->getline()) ) {
+	$self->{errors} .= time2str("[%Y-%m-%d %H:%M:%S] ",time,'GMT') if $self->{timestamps};
 	$self->{errors} .= $_;
     }
 
