@@ -1,7 +1,7 @@
 #
-# $Id: FS.pm,v 2.6 2003/09/29 18:08:10 wpm Exp $
+# $Id$
 #
-# (c) 2003 Morgan Stanley and Co.
+# (c) 2003-2004 Morgan Stanley and Co.
 # See ..../src/LICENSE for terms of distribution.
 #
 
@@ -21,7 +21,7 @@ use AFS::Object::Server;
 use AFS::Object::ACL;
 
 our @ISA = qw(AFS::Command::Base);
-our $VERSION = '1.1';
+our $VERSION = '1.9';
 
 sub checkservers {
 
@@ -282,8 +282,8 @@ sub _paths_method {
 		my ($volname,$quota,$used,$percent,$partition) = split;
 
 		$quota = 0 if $quota eq "nolimit";
-		$percent =~ s/%//g; # Don't need it -- want numeric result
-		$partition =~ s/%//g; # Don't need it -- want numeric result
+		$percent =~ s/\D//g; # want numeric result
+		$partition =~ s/\D//g; # want numeric result
 
 		$path->_setAttribute
 		  (
