@@ -1,7 +1,7 @@
 #
-# $Id: VOS.pm,v 3.2 2003/10/08 21:58:22 wpm Exp $
+# $Id$
 #
-# (c) 2003 Morgan Stanley and Co.
+# (c) 2003-2004 Morgan Stanley and Co.
 # See ..../src/LICENSE for terms of distribution.
 #
 
@@ -23,7 +23,7 @@ use AFS::Object::Partition;
 use AFS::Object::Transaction;
 
 our @ISA = qw(AFS::Command::Base);
-our $VERSION = '1.2';
+our $VERSION = '1.9';
 
 sub examine {
 
@@ -47,6 +47,12 @@ sub examine {
 
 	chomp;
 
+	#
+	# These two lines are part of the verbose output
+	#
+	next if /Fetching VLDB entry/;
+	next if /Getting volume listing/;
+
 	#
 	# This code parses the volume header information.  If we match
 	# this line, then we go after the information we expect to be
@@ -54,38 +60,78 @@ sub examine {
 	# might very well have several of these chunks of data for RO
 	# volumes.
 	#
-	if ( /^$args{id}\s+(\d+)\s+(RW|RO|BK)\s+(\d+)\s+K/ ) {
+	if ( /^\*{4}/ ) {
 
 	    my $header = AFS::Object::VolumeHeader->new();
 
-	    if ( /^$args{id}\s+(\d+)\s+(RW|RO|BK)\s+(\d+)\s+K\s+([\w-]+)/ ) {
-
+	    if ( /Volume (\d+) is busy/ ) {
 		$header->_setAttribute
 		  (
-		   name			=> $args{id},
-		   id 			=> $1,
-		   type 		=> $2,
-		   size 		=> $3,
-		   status 		=> $4,
-		  );
-		$header->_setAttribute( rwrite	=> $1 ) if $2 eq 'RW';
-		$header->_setAttribute( ronly	=> $1 ) if $2 eq 'RO';
-		$header->_setAttribute( backup	=> $1 ) if $2 eq 'BK';
-
-	    } elsif ( /^$args{id}\s+(\d+)\s+(RW|RO|BK)\s+(\d+)\s+K\s+used\s+(\d+)\s+files\s+([\w-]+)/ ) {
-
-		$header->_setAttribute
-		  (
-		   name			=> $args{id},
 		   id			=> $1,
-		   type 		=> $2,
-		   size 		=> $3,
-		   files 		=> $4,
-		   status 		=> $5,
+		   status		=> 'busy',
+		   attached		=> 1,
+		  );
+	    } elsif ( /Could not attach volume (\d+)/ ) {
+		$header->_setAttribute
+		  (
+		   id			=> $1,
+		   status		=> 'offline',
+		   attached		=> 0,
+		  );
+	    }
+
+	    $result->_addVolumeHeader($header);
+
+	    next;
+
+	} elsif ( /^(\S+)\s+(\d+)\s+(RW|RO|BK)\s+(\d+)\s+K/ ) {
+
+	    my $header = AFS::Object::VolumeHeader->new();
+
+	    if ( /^(\S+)\s+(\d+)\s+(RW|RO|BK)\s+(\d+)\s+K\s+([\w-]+)/ ) {
+
+		$header->_setAttribute
+		  (
+		   name			=> $1,
+		   id 			=> $2,
+		   type 		=> $3,
+		   size 		=> $4,
+		  );
+		$header->_setAttribute( rwrite	=> $2 ) if $3 eq 'RW';
+		$header->_setAttribute( ronly	=> $2 ) if $3 eq 'RO';
+		$header->_setAttribute( backup	=> $2 ) if $3 eq 'BK';
+
+		my $status = $5;
+		$status = 'offline' if $status eq 'Off-line';
+		$status = 'online' if $status eq 'On-line';
+		$header->_setAttribute
+		  (
+		   status 		=> $status,
+		   attached		=> 1,
+		  );
+
+	    } elsif ( /^(\S+)\s+(\d+)\s+(RW|RO|BK)\s+(\d+)\s+K\s+used\s+(\d+)\s+files\s+([\w-]+)/ ) {
+
+		$header->_setAttribute
+		  (
+		   name			=> $1,
+		   id			=> $2,
+		   type 		=> $3,
+		   size 		=> $4,
+		   files 		=> $5,
+		  );
+		$header->_setAttribute( rwrite	=> $2 ) if $3 eq 'RW';
+		$header->_setAttribute( ronly	=> $2 ) if $3 eq 'RO';
+		$header->_setAttribute( backup	=> $2 ) if $3 eq 'BK';
+
+		my $status = $6;
+		$status = 'offline' if $status eq 'Off-line';
+		$status = 'online' if $status eq 'On-line';
+		$header->_setAttribute
+		  (
+		   status 		=> $status,
+		   attached		=> 1,
 		  );
-		$header->_setAttribute( rwrite	=> $1 ) if $2 eq 'RW';
-		$header->_setAttribute( ronly	=> $1 ) if $2 eq 'RO';
-		$header->_setAttribute( backup	=> $1 ) if $2 eq 'BK';
 
 	    } else {
 
@@ -146,13 +192,28 @@ sub examine {
 		    next;
 		}
 
+		if ( /^\s+Copy\s+(.*)\s*$/ ) {
+		    $header->_setAttribute( copyTime	=> $1 );
+		    next;
+		}
+
+		if ( /^\s+Backup\s+(.*)\s*$/ ) {
+		    $header->_setAttribute( backupTime	=> $1 );
+		    next;
+		}
+
+		if ( /^\s+Last Access\s+(.*)\s*$/ ) {
+		    $header->_setAttribute( access	=> $1 );
+		    next;
+		}
+
 		if ( /^\s+Last Update\s+(.*)\s*$/ ) {
 		    $header->_setAttribute( update	=> $1 );
 		    next;
 		}
 
-		if ( /^\s+(\d+) access/ ) {
-		    $header->_setAttribute( access	=> $1 );
+		if ( /^\s+(\d+) accesses/ ) {
+		    $header->_setAttribute( accesses	=> $1 );
 		    next;
 		}
 
@@ -308,8 +369,6 @@ sub examine {
 
 	    }
 
-	    next;
-
 	}
 
 	#
@@ -660,6 +719,7 @@ sub listvol {
 		  (
 		   id			=> $1,
 		   status		=> 'busy',
+		   attached		=> 1,
 		  );
 		$partition->_addVolumeHeader($volume);
 		next;
@@ -668,6 +728,7 @@ sub listvol {
 		  (
 		   id			=> $1,
 		   status		=> 'offline',
+		   attached		=> 0,
 		  );
 		$partition->_addVolumeHeader($volume);
 		next;
@@ -686,6 +747,8 @@ sub listvol {
 
 	    if ( @array == 6 ) {
 		($name,$id,$type,$size,$status) = @array[0..3,5];
+		$status = 'offline' if $status eq 'Off-line';
+		$status = 'online' if $status eq 'On-line';
 		$volume->_setAttribute
 		  (
 		   id			=> $id,
@@ -693,12 +756,14 @@ sub listvol {
 		   type			=> $type,
 		   size			=> $size,
 		   status		=> $status,
+		   attached		=> 1,
 		  );
 	    } elsif ( @array == 1 ) {
 		$volume->_setAttribute
 		  (
 		   id			=> $_,
 		   status		=> 'online',
+		   attached		=> 1,
 		  );
 	    } else {
 		$self->_Carp("Unable to parse header summary line:\n" . $_);
@@ -741,13 +806,28 @@ sub listvol {
 			next;
 		    }
 
+		if ( /^\s+Copy\s+(.*)\s*$/ ) {
+		    $volume->_setAttribute( copyTime	=> $1 );
+		    next;
+		}
+
+		if ( /^\s+Backup\s+(.*)\s*$/ ) {
+		    $volume->_setAttribute( backupTime	=> $1 );
+		    next;
+		}
+
+		if ( /^\s+Last Access\s+(.*)\s*$/ ) {
+		    $volume->_setAttribute( access	=> $1 );
+		    next;
+		}
+
 		    if ( /^\s+Last Update\s+(.*)\s*$/ ) {
 			$volume->_setAttribute( update => $1 );
 			next;
 		    }
 
-		    if ( /^\s+(\d+) access/ ) {
-			$volume->_setAttribute( access => $1 );
+		    if ( /^\s+(\d+) accesses/ ) {
+			$volume->_setAttribute( accesses => $1 );
 			next;
 		    }
 		}		# while(defined($_ = $self->{handle}->getline())) {
@@ -924,12 +1004,13 @@ sub dump {
     my $gzip_default = 6;
     my $bzip2_default = 6;
 
+    my $nocompress = delete $args{nocompress} || undef;
     my $gzip = delete $args{gzip} || undef;
     my $bzip2 = delete $args{bzip2} || undef;
     my $filterout = delete $args{filterout} || undef;
 
-    if ( $gzip && $bzip2 ) {
-	$self->_Carp("Invalid argument combination: only one of 'gzip' or 'bzip2' may be specified");
+    if ( $gzip && $bzip2 && $nocompress ) {
+	$self->_Carp("Invalid argument combination: only one of 'gzip' or 'bzip2' or 'nocompress' may be specified");
 	return;
     }
 
@@ -940,9 +1021,9 @@ sub dump {
 
     if ( $file ne 'stdout' ) {
 
-	if ( $file =~ /\.gz$/ && not defined $gzip ) {
+	if ( $file =~ /\.gz$/ && not defined $gzip and not defined $nocompress ) {
 	    $gzip = $gzip_default;
-	} elsif ( $file =~ /\.bz2$/ && not defined $bzip2 ) {
+	} elsif ( $file =~ /\.bz2$/ && not defined $bzip2 and not defined $nocompress ) {
 	    $bzip2 = $bzip2_default;
 	}
 
@@ -1018,12 +1099,13 @@ sub restore {
 	return;
     };
 
+    my $nocompress = delete $args{nocompress} || undef;
     my $gunzip = delete $args{gunzip} || undef;
     my $bunzip2 = delete $args{bunzip2} || undef;
     my $filterin = delete $args{filterin} || undef;;
 
-    if ( $gunzip && $bunzip2 ) {
-	$self->_Carp("Invalid argument combination: only one of 'gunzip' or 'bunzip2' may be specified");
+    if ( $gunzip && $bunzip2 && $nocompress ) {
+	$self->_Carp("Invalid argument combination: only one of 'gunzip' or 'bunzip2' or 'nocompress' may be specified");
 	return;
     }
 
@@ -1034,9 +1116,9 @@ sub restore {
 
     if ( $file ne 'stdin' ) {
 
-	if ( $file =~ /\.gz$/ && not defined $gunzip ) {
+	if ( $file =~ /\.gz$/ && not defined $gunzip and not defined $nocompress ) {
 	    $gunzip = 1;
-	} elsif ( $file =~ /\.bz2$/ && not defined $bunzip2 ) {
+	} elsif ( $file =~ /\.bz2$/ && not defined $bunzip2 and not defined $nocompress ) {
 	    $bunzip2 = 1;
 	}
 
