@@ -1,7 +1,7 @@
 #
-# $Id: ACL.pm,v 2.2 2003/09/02 21:27:08 wpm Exp $
+# $Id$
 #
-# (c) 2003 Morgan Stanley and Co.
+# (c) 2003-2004 Morgan Stanley and Co.
 # See ..../src/LICENSE for terms of distribution.
 #
 
@@ -10,7 +10,7 @@ package AFS::Object::ACL;
 use strict;
 
 our @ISA = qw(AFS::Object);
-our $VERSION = '1.1';
+our $VERSION = '1.9';
 
 sub getPrincipals {
     my $self = shift;
@@ -22,7 +22,7 @@ sub getRights {
     my $self = shift;
     my $principal = shift;
     return unless ref $self->{_principals};
-    return $self->{_principals}->{$principal};
+    return $self->{_principals}->{lc($principal)};
 }
 
 sub getEntries {
